source [file join [file dirname [info script]] mlx_i2c.tcl]
global DZ

proc Set_RegTable {} {
    global RegTable DZ
    catch {unset RegTable}
    array set RegTable {
        Temp   "0 none Temperature"
        Conf   "0 none Configuration"
        T_OS   "0 none Set Point HYST"
        T_HYST "0 none Set Point OS"
    }
}

proc List {_slave} {
    global RegTable DZ
    foreach name "Temp Conf T_HYST T_OS" {
        Inform "-I-dump:data" -data "$name"
    }
}

proc Query {_slave {dump 0}} {
    global RegTable DZ
    set res ""
    foreach name "Temp Conf T_HYST T_OS" {
        set lvl [lindex $RegTable($name) 0]
        if {$lvl} {
            continue
        }
        set address [lindex $RegTable($name) 1]
        set desc [lrange $RegTable($name) 2 end]
        set val [Read $address $_slave $name]
        lappend res "$name,$val"
        if {$dump} {
            Inform "-I-dump:data" -data "[SeperateBySpaces "$name = $val [list $desc]" 20]"
        }
    }
    return $res
}

proc Read_Handle {_list_read_address _i2c_address} {
    global RegTable DZ
    if {$_list_read_address == "all"} {
        foreach name [array names RegTable] {
            set lvl [lindex $RegTable($name) 0]
            set address [lindex $RegTable($name) 1]
            set desc [lrange $RegTable($name) 2 end]
            set val [Read $address $_i2c_address $name]
            Inform "-I-dump:data" -data "$name = $val"
        }
        return 0
    } elseif {[lsearch [array names RegTable] $_list_read_address] != -1} {
        set name $_list_read_address
        set address [lindex $RegTable($name) 1]
        set val [Read $address $_i2c_address $name]
        if {$val != -1} {
            Inform "-I-dump:data" -data "$name = $val"
        }
    } elseif {$_list_read_address == 0} {
        set name "Pointer"
        set address 0
        set val [Read $address $_i2c_address $name]
        if {$val != -1} {
            Inform "-I-dump:data" -data "$name = $val"
        }
    } else {
        Inform "-E-argv:invalid.read.address" -value $_list_read_address
    }
}

proc Write_Handle {_list_write_address_vals _i2c_address} {
    foreach pair [split $_list_write_address_vals ,] {
        if {[regexp {^([^=]+)=([0-9a-f0-F]+)$} $pair . reg val]} {
            if {[string is integer $reg]} {
                set name [Get_NameByReg $reg]
                set address $reg
                Write $address $val $_i2c_address $name 
            } elseif {[Get_RegByName $reg] != -1 } {
                set name $reg
                set address [Get_RegByName $reg]
                Write $address $val $_i2c_address $name 
            } else {
                Inform "-E-argv:invalid.write.param" -value $pair
            }
        } else {
            Inform "-E-argv:invalid.write.param" -value $pair
        }
    }
}


proc Read {_address _i2c_address _name} {
    global DZ

    if {$_address == "none"} {
        set pointer_table(Temp) 0
        set pointer_table(Conf) 1
        set pointer_table(T_HYST) 2
        set pointer_table(T_OS) 3

        switch $_name {
            "T_OS" -
            "T_HYST" -
            "Temp" {
                set args_list "-dev $DZ(argv:mst.device) -i2c_address $_i2c_address -address $pointer_table($_name)"
                set val [I2C_Read [concat $args_list "-x 2 -a 1"]]
                if {$val == -1} { return -1 }
                # remove the 7 unneeded Bits
                set val [expr $val / 128]
                # Change into temp
                set val "[expr ${val}.0 / 2]c"
                return $val
            }
            "Conf" {
                set args_list "-dev $DZ(argv:mst.device) -i2c_address $_i2c_address -address $pointer_table($_name)"
                return [I2C_Read [concat $args_list "-x 1 -a 1"]]    
            }
            default {
                return -1
            }
        }
    } elseif {$_address == 0} {
        set args_list "-dev $DZ(argv:mst.device) -i2c_address $_i2c_address -address 0x0"
        return [I2C_Read $args_list]    
    }
    return -1
}

proc Write {_address _val _i2c_address _name} {
    global DZ

    if {$_address == "none"} {
        set pointer_table(Temp) 0
        set pointer_table(Conf) 1
        set pointer_table(T_HYST) 2
        set pointer_table(T_OS) 3

        switch $_name {
            "T_OS" -
            "T_HYST" -
            "Temp" {
                set _val [expr $_val * 128 * 2]
                set args_list "-dev $DZ(argv:mst.device) -i2c_address $_i2c_address -address 0x0"
                I2C_Write [concat $args_list "-data $pointer_table($_name) -a 0"]
                return [I2C_Write [concat $args_list "-data $_val -a 0"]]
            }
            "Conf" {
                set args_list "-dev $DZ(argv:mst.device) -i2c_address $_i2c_address -address 0x0"
                I2C_Write [concat $args_list "-data $pointer_table($_name) -a 0"]
                return [I2C_Write [concat args_list "-data $_val -a 0"]]
            }
            default {
                return -1
            }
        }
    } elseif {$_address == 0} {
        set args_list "-dev $DZ(argv:mst.device) -i2c_address $_i2c_address -address 0x0"
        return [I2C_Write [concat $args_list "-data $_val -a 0"]]
    }
    return -1
}

proc Get_NameByReg {_address} {
    global RegTable
    foreach name [array names RegTable] {
        set val [lindex $RegTable($name) 1]
        if {$val == $_address} {
            return $name
        }
    }
    return "<No Name>"
}

proc Get_RegByName {_name} {
    global RegTable
    if {![info exists RegTable($_name)]} {
        puts "NONE VALID NAME $_name"
        return -1
    }
    return [lindex $RegTable($_name) 1]
}

Set_RegTable

if {[info exists DZ(var:tool.name)]} {
    return 0
}

### Pre runs
Initialize_MlxI2C
Start_MlxI2C

set mode ""
if {[info exists DZ(argv:read.address)]} {
    set mode _Read
}
if {[info exists DZ(argv:write.address.val)]} {
    set mode _Write
}
if {[info exists DZ(argv:list)]} {
    set mode _List
}
if {[info exists DZ(argv:query)]} {
    set mode _Query
}

Inform "-I-sharq:Mode.header" -mode $mode

switch $mode {
    "_List" {
        List $DZ(argv:slave.address)
    }
    "_Query" {
        Query $DZ(argv:slave.address) 1
    }
    "_Read" {
        Read_Handle $DZ(argv:read.address) $DZ(argv:slave.address)
    }
    "_Write" {
        Write_Handle $DZ(argv:write.address.val) $DZ(argv:slave.address)
    }
}

Finish_MlxI2C
